@@ -17,5 +17,11 @@ void info_Stein(double *p, int *maxdim, int *CEbadlybehaved);
 int checkNinit_Stein(covinfo_arraytype keycov, covlist_type covlist, 
 		     int remaining_covlistlength, SimulationType method);
 
+double MaStein(double *x, double *p, int effectivedim);
+void range_MaStein(int dim, int *index, double* range);
+void info_MaStein(double *p, int *maxdim, int *CEbadlybehaved);
+int checkNinit_MaStein(covinfo_arraytype keycov, covlist_type covlist, 
+		       int remaining_covlistlength, SimulationType method);
+
 #endif /* Hyperfcts_H */
  
